# -*- coding: utf-8 -*-


from .version import version as __version__

__all__ = [
    'beatles',
    'beatport_key',
    'dali',
    'giantsteps_tempo',
    'giantsteps_key',
    'groove_midi',
    'gtzan_genre',
    'guitarset',
    'ikala',
    'maestro',
    'medley_solos_db',
    'medleydb_melody',
    'medleydb_pitch',
    'mridangam_stroke',
    'orchset',
    'rwc_classical',
    'rwc_jazz',
    'rwc_popular',
    'salami',
    'tinysol',
]
